//---------------------------------------------------------------------------
#include <iostream.h>
#include <fstream.h>
#include "VectorException.h"

template <class ElemType>
class Vector {
  protected:
    ElemType *data;
    int position;
    int allocated;
    double GrowFactor;
  public:
    Vector(int dim=10);
    Vector(const Vector<ElemType>& other);
    virtual ~Vector();
    virtual int Size()const;
    virtual int Elements() const;
    virtual bool IsEmpty() const;
    virtual bool IsFull() const;
    virtual void Insert(const ElemType& e);
    virtual void InsertAt(int pos,const ElemType& e);
    virtual ElemType Get() const;
    virtual ElemType GetAt(int pos) const;
    virtual ElemType& operator[](int index);
    virtual bool Has(const ElemType& e) const;
    virtual ElemType Remove();
    virtual bool Remove(const ElemType& e);
    //--------------------------------------------
    virtual void Write(ostream& out=cout);
    virtual void WriteLine(ostream& out=cout);
    virtual bool Read(istream& in=cin);
    virtual void Append(istream& in=cin);
    virtual void WriteBin(ostream& out=cout);
    virtual void ReadBin(istream& in=cin);
    virtual void AppendBin(istream& in=cin);
    //--------------------------------------------
    virtual void Write(char* file);
    virtual void WriteLine(char* file);
    virtual bool Read(char* file);
    virtual void Append(char* file);
    virtual void WriteBin(char* file);
    virtual void ReadBin(char* file);
    virtual void AppendBin(char* file);

    //------------------Implementacao alunos ---------------
    void SetGrowFactor(double f);
    virtual int IndexOf(const ElemType& e);
    virtual ElemType RemoveAt(int p);
    virtual void RemoveAll(const ElemType& e);
    virtual void Subst(const ElemType& e, const ElemType& f);
    virtual void SubstAll(const ElemType& e, const ElemType& f);
    virtual void Erase(int p1, int p2);
    virtual void Clear();
    virtual void Reverse();
    virtual void Trunc();
    virtual void Resize(int s);
    virtual bool operator==(const Vector<ElemType>& other) const;
    virtual bool operator!=(const Vector<ElemType>& other) const;
    virtual Vector<ElemType>& operator=(const Vector<ElemType>& other);
    virtual void Copy(const Vector<ElemType>& other);

  protected:

    void Expand();
    void ExpandTo(int newSize);
};


//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// Cdigo fornecido
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Construtor que aceita como argumento o tamanho inicial do array
//---------------------------------------------------------------------------
template <class ElemType>
Vector<ElemType>::Vector(int dim) {
  data = new ElemType[dim];
  allocated=dim;
  position=0;
  GrowFactor=2.0;
}
//---------------------------------------------------------------------------
// Construtor Cpia
//---------------------------------------------------------------------------
template <class ElemType>
Vector<ElemType>::Vector(const Vector<ElemType>& other) {
  data = new ElemType [other.position];
  allocated = other.position;
  position = other.position;
  growFactor= other.growFactor;
  for(int i=0; i< other.position; i++)
    data[i] = other.data[i];
}
//---------------------------------------------------------------------------
template <class ElemType>
Vector<ElemType>::~Vector() {
  delete []data;
}
//---------------------------------------------------------------------------
template <class ElemType>
int Vector<ElemType>::Size()const {
  return allocated;
}
//---------------------------------------------------------------------------
template <class ElemType>
int Vector<ElemType>::Elements() const {
  return position;
}
//---------------------------------------------------------------------------
template <class ElemType>
bool Vector<ElemType>::IsEmpty() const {
  return position==0;
}
//---------------------------------------------------------------------------
template <class ElemType>
bool Vector<ElemType>::IsFull() const {
  return position==allocated;
}
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::Insert(const ElemType& e) {
  if( IsFull())
    Expand();
  data[position++] = e;
}
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::InsertAt(int pos,const ElemType& e) {
  if( pos < 0 )
    throw VectorException("Insert index ERROR: less than zero");
  if( pos >= allocated)
    ExpandTo(pos+1);
  data[pos]=e;
  position=pos;
}
//---------------------------------------------------------------------------
// pre: ! empty()
// pos: return last element
//---------------------------------------------------------------------------
template <class ElemType>
ElemType Vector<ElemType>::Get() const {
  if( position == 0 )
    throw VectorException("Get Error : Vector empty");
  return data[position-1];
}
//---------------------------------------------------------------------------
// pre: pos >=0 && pos < Elements();
// pos: returns  element at position pos
//---------------------------------------------------------------------------
template <class ElemType>
ElemType Vector<ElemType>::GetAt(int pos) const {
  if( pos < 0 || pos >= position )
    throw VectorException("Get index ERROR: invalid index");
  return data[pos];
}

//---------------------------------------------------------------------------
//---------   Dynamic   Memory          -------------------------------------
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::Expand() {
  //Alocar novo array dinmico
  ElemType *dataTmp = new ElemType [GrowFactor*allocated];//Alteracao para GrowFactor
  //Copiar os elementos do array antigo
  for(int i=0; i< allocated; i++)
    dataTmp[i]=data[i];
  //actualizar o nmero de elementos alocados
  allocated *=2;
  // libertar a memria do array antigo
  delete [] data;
  // actualizar o ponteiro de dados
  data= dataTmp;
}
//---------------------------------------------------------------------------
// pre: newSize > allocated
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::ExpandTo(int newSize) {
  //Alocar novo array dinmico
  ElemType *dataTmp = new ElemType [newSize];
  //Copiar os elementos do array antigo
  for(int i=0; i< allocated; i++)
    dataTmp[i]=data[i];
  //actualizar o nmero de elementos alocados
  allocated = newSize;
  // libertar a memria do array antigo
  delete [] data;
  // actualizar o ponteiro de dados
  data= dataTmp;
}

//---------------------------------------------------------------------------
//--------         operators          ---------------------------------------
//---------------------------------------------------------------------------

// pre: pos >=0 && pos < Elements();
// pos: returns or set  element at position index
template <class ElemType>
ElemType& Vector<ElemType>::operator[](int index) {
  if( index < 0 || index >= allocated)
    throw VectorException("ERROR: index out of range");
  return data[index];
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
template <class ElemType>
bool Vector<ElemType>::Has(const ElemType& e) const {
  for(int i=0; i< Elements(); i++)
    if ( data[i] == e)
      return true;
   return false;
}

//---------------------------------------------------------------------------
// pre: ! IsEmpty()
// pos: remove and return last element
template <class ElemType>
ElemType Vector<ElemType>::Remove() {
  if (IsEmpty())
    throw VectorException("Vector is empty");
  return data[--position];
}

//---------------------------------------------------------------------------
// pre: Has(e)
// pos: remove and return element e
template <class ElemType>
bool Vector<ElemType>::Remove(const ElemType& e) {
  if ( !Has(e))
    return false;
  int index;
  for(index=0; index< position; index++)
       if ( data[index] == e)
         break;
  for( index++ ; index< position; index++)
     data[index-1] = data[index];
  position--;
  return true;
}

//---------------------------------------------------------------------------
//--- Ficha 3 ----- Implementaao
//---------------------------------------------------------------------------
//

//--  Write -----------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::Write(ostream& out) {
  for(int i=0 ; i < Elements() ; i++ )
    out << data[i] << " ";
}

//--  WriteLine -------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::WriteLine(ostream& out) {
  Write(out);
  out << endl;
}

//--  Read ------------------------------------------------------------------
template <class ElemType>
bool Vector<ElemType>::Read(istream& in) {

  if( in ) {
      ElemType aux;
      this->Clear();
      while( true )
      {
        in >> aux ;
        if( in.eof() )
           break;
           Insert(aux);
      }
      return true;
  }
  else
    return false;

}

//--  Append ------------------------------------------------------------------

template <class ElemType>
void Vector<ElemType>::Append(istream& in) {

  if( in ) {
      ElemType aux;
      while( true )
      {
        in >> aux ;
        Insert(aux);
        if( in.eof() )
           break;

      }
  }
  else
    cout << "ERRO";

}

//--  WriteBin -----------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::WriteBin(ostream& out) {
  for(int i=0 ; i < position ; i++ )
    out.write((char *)&data[i],sizeof(ElemType)) ;
}

//--  ReadBin -----------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::ReadBin(istream& in) {
this->Clear();
while( in.good() )
  {
    ElemType aux;
    in.read((char *) &aux , sizeof(ElemType));
    Insert(aux);
  }
}

//--  AppendBin -----------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::AppendBin(istream& in) {
while( in.good() )
  {
    ElemType aux;
    in.read((char *) &aux , sizeof(ElemType));
    Insert(aux);
  }
}

//--  Write -----------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::Write(char* file) {
  ofstream out(file);
  Write(out);
  out.close();
}

//--  WriteLine -------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::WriteLine(char* file) {
  ofstream out(file);
  WriteLine(out);
  out.close();
}

//--  Read ------------------------------------------------------------------
template <class ElemType>
bool Vector<ElemType>::Read(char* file) {
  ifstream in(file);
  return Read(in);
}

//--  Append ------------------------------------------------------------------

template <class ElemType>
void Vector<ElemType>::Append(char* file) {
  ifstream in(file);
  Append(in);
  in.close();
}

//--  WriteBin -----------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::WriteBin(char* file) {
  ofstream out(file,ios::out | ios::binary);
  WriteBin(out);
  out.close();
}

//--  ReadBin -----------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::ReadBin(char* file) {
  ifstream in(file,ios::in | ios::binary);
  ReadBin(in);
  in.close();
}

//--  AppendBin -----------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::AppendBin(char* file) {
  ifstream in(file,ios::in | ios::binary);
  AppendBin(in);
  in.close();
}










//---------------------------------------------------------------------------
// Cdigo implementado
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//Ex1
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::SetGrowFactor(double f){
   GrowFactor=f;
}
//---------------------------------------------------------------------------
//Ex2
//---------------------------------------------------------------------------

template <class ElemType>
int Vector<ElemType>::IndexOf(const ElemType& e) {
   for(int index=0; index<position; index++)
      if (e==data[index])
         return index;
   return -1;
}
//---------------------------------------------------------------------------
//Ex3
//---------------------------------------------------------------------------
template <class ElemType>
ElemType Vector<ElemType>::RemoveAt(int p) {
   //codigo que remove o 1 valor do vector igual ao p
   //return Remove(data[p]);

   if (p<0 || p>position-1)
      throw VectorException("Position Error!");
   ElemType aux=data[p];

   for(int i=p; i< position; i++)
      data[i]=data[i+1];
   position--;

   /*ElemType *dataTmp = new ElemType [--position];
   for(int i=0; i< position; i++)
      if (i<p)
         dataTmp[i]=data[i];
      else
         dataTmp[i]=data[i+1];

   allocated = position;
   delete [] data;
   data= dataTmp;*/

   return aux;

}
//---------------------------------------------------------------------------
//Ex4
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::RemoveAll(const ElemType& e) {
   while (Has(e))
      Remove(e);
}
//---------------------------------------------------------------------------
//Ex5
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::Subst(const ElemType& e, const ElemType& f) {
   if ( !Has(e))
      throw VectorException(" Element not found");
   data[IndexOf(e)]=f;
}
//---------------------------------------------------------------------------
//Ex6
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::SubstAll(const ElemType& e, const ElemType& f) {
   if ( !Has(e))
      throw VectorException(" Element not found");
   while (Has(e))
      Subst(e,f);
}
//---------------------------------------------------------------------------
//Ex7
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::Erase(int p1, int p2) {
   if ((p1 >=0 && p1<position) && (p2 >=p1 && p2<position))
      for(;p1<=p2;p2--)
         RemoveAt(p1);
   else
      throw VectorException("Position Error!");
}
//---------------------------------------------------------------------------
//Ex8
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::Clear() {
    position=0;
}
//---------------------------------------------------------------------------
//Ex9
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::Reverse() {

   ElemType *dataTmp = new ElemType [position];
   allocated=position;
   for(int i=position-1; i>=0; i--)
      dataTmp[position-1-i]=data[i];
   delete [] data;
   data= dataTmp;

   /*ElemType aux;
   for(int i=0;i<position/2 ; i++)
   {
      aux = data[i];
      data[i] = data[position-1-i];
      data[position-1-i] = aux;
   }*/

}
//---------------------------------------------------------------------------
//Ex10
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::Trunc() {
ElemType *dataTmp = new ElemType [position];
   allocated=position;
   for(int i=0; i<position; i++)
      dataTmp[i]=data[i];
   delete [] data;
   data= dataTmp;
}
//---------------------------------------------------------------------------
//Ex11
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::Resize(int s) {

   if (s >= position){
      if( s>allocated)
         ExpandTo(s);
      else{
         ElemType *dataTmp = new ElemType [s];
         allocated=s;
         for(int i=0; i< s; i++)
         dataTmp[i]=data[i];
         delete [] data;
         data= dataTmp;
      }
   }
   else
      Trunc();

}
//---------------------------------------------------------------------------
//Ex12
//---------------------------------------------------------------------------
template <class ElemType>
bool Vector<ElemType>::operator ==(const Vector<ElemType>& other) const {
   if (other.position!=position)
      return false;
   else
      for(int index=0; index<position; index++)
         if (other.data[index]!=data[index])
            return false;
   return true;
}

template <class ElemType>
bool Vector<ElemType>::operator !=(const Vector<ElemType>& other) const {
   return ! (*this == other);
   //return ! operator == (other);
}
//---------------------------------------------------------------------------
//Ex13
//---------------------------------------------------------------------------
template <class ElemType>
void Vector<ElemType>::Copy(const Vector<ElemType>& other){
   if (this != &other){
      delete [] data;
      data = new ElemType[other.position];
      position =allocated=other.position;
      for (int i = 0; i < other.position; i++)
         data[i] = other.data[i];
   }
}

template <class ElemType>
Vector<ElemType>& Vector<ElemType>::operator=(const Vector<ElemType>& other){
   Copy(other);
   return *this;
}
